#ifndef _POLYHEDRON_H_
#define _POLYHEDRON_H_


//////////////////////////////////////////////////////////////////////////////
// Includes

#include "util/linear.h"
#include <vector>


//////////////////////////////////////////////////////////////////////////////
// struct Polyhedron
// This class implements a dual 3-simplex; the polyhedron associated with a
// mesh vertex. These are convex and every vertex is on exactly three planes.

struct Polyhedron
{
    void BarycentricCoords( const vec3& p, std::vector<real>& weights );
    void CalculateCoefficients( void );

    struct PolyhedronVertex
    {
        // The world space position of this vertex;
        vec3 position;

        // The tet whose circumcenter corresponds to this vertex
        size_t tetID;

        // The three planes that touch this vertex.
        // These ints index into m_planeNormals.
        vec<3,int> planes;

        // The volume spanned by this vertex's three plane normals.
        // Used for finding generalized barycentric coordinates.
        real V;
    };

    std::vector<PolyhedronVertex> m_vertices;
    std::vector<vec3> m_planeNormals;
};


#endif  // _POLYHEDRON_H_
