#ifndef _SIMPLICIAL_COMPLEX_FACE_H_
#define _SIMPLICIAL_COMPLEX_FACE_H_


//////////////////////////////////////////////////////////////////////////////
// Includes

#include "util/linear.h"


//////////////////////////////////////////////////////////////////////////////

template<int N>
class SimplicialComplex;
class Vertex;
class Edge;


//////////////////////////////////////////////////////////////////////////////
// class Face
// A class for doing operations on and querying a 2-simplex of a complex

class Face
{
    // friends
    template<typename T>
    friend class SimplexIterator;
    friend class EdgeFaceIterator;
    friend class SimplicialComplex;
    friend class Tet;

    friend bool operator==( const Face&, const Face& );
    friend bool operator!=( const Face&, const Face& );

public:  // methods
    Face();
    ~Face();

    operator bool() const;

    size_t Index( void ) const  { return index; }

    vec3 Center    ( void ) const;
    real Area      ( void ) const;
    real DualLength( void ) const;
    vec3 Normal    ( void ) const;

    void Vertices( vec3& v0, vec3& v1, vec3& v2 ) const;
    void Vertices( Vertex& v0, Vertex& v1, Vertex& v2 ) const;
    void Edges( Edge& e0, Edge& e1, Edge& e2 ) const;
    void Tets( Tet& t0, Tet& t1 ) const;

    bool IsOnBoundary( void ) const;

private:  // methods
    Face( const SimplicialComplex<3> *sc, size_t index );

private:  // data
    const SimplicialComplex<3> *sc;
    size_t index;
};


#endif  // _SIMPLICIAL_COMPLEX_FACE_H_
