#ifndef _SIMPLICIAL_COMPLEX_IO_H_
#define _SIMPLICIAL_COMPLEX_IO_H_

#include <iostream>


template<int N>
class SimplicialComplex;

/** @addtogroup mesh_io */
// @{

/** Mesh file types. */
typedef enum { AUTO, OBJ, OFF, TET } FileType;

/**
* Returns the mesh file type identified by the specified filename.  If the
* file type is unrecognized, then @c AUTO is returned.
* @param filename  filename
* @return file type
*/
FileType GetFileType( const char* filename );

/**
* Reads a mesh from the specified input stream.
* @param mesh  mesh
* @param in    input stream
* @param type  file type
* @return true if read was successful
*/
bool Read( SimplicialComplex<2>& mesh, std::istream& in, FileType type );
bool Read( SimplicialComplex<3>& mesh, std::istream& in, FileType type );

/**
* Reads a mesh from the file with the specified filename.
* @param mesh  mesh
* @param filename  filename
* @param type  file type
* @return true if read was successful
*/
template<int N>
bool Read( SimplicialComplex<N>& mesh, const char* filename, FileType type = AUTO );

// explicit instantiation
//template bool Read<2>( SimplicialComplex<2>& mesh, const char* filename, FileType type );
template bool Read<3>( SimplicialComplex<3>& mesh, const char* filename, FileType type );


// @}


#endif  // _SIMPLICIAL_COMPLEX_IO_H_
