#ifndef _GEOMETRY_H_
#define _GEOMETRY_H_


//////////////////////////////////////////////////////////////////////////////
// Includes

#include "util/linear.h"
#include <vector>


//////////////////////////////////////////////////////////////////////////////

// Circumcenter - 2D
vec2 Circumcenter( const vec2& v1, const vec2& v2, const vec2& v3 );

// Circumcenter - 3D
vec3 Circumcenter( const vec3& v1, const vec3& v2, const vec3& v3 );

// Circumcenter - 3D
vec3 Circumcenter( const vec3& v1, const vec3& v2, const vec3& v3, const vec3& v4 );

// Barycenter - 2D
vec2 Barycenter( const vec2& v1, const vec2& v2, const vec2& v3 );

// Barycenter - 3D
vec3 Barycenter( const vec3& v1, const vec3& v2, const vec3& v3, const vec3& v4 );

// Length - of a line segment in 2D
real Length( const vec2& v1, const vec2& v2 );

// Length - of a line segment in 3D
real Length( const vec3& v1, const vec3& v2 );

// Area - of a triangle in 2D
real Area( const vec2& v1, const vec2& v2, const vec2& v3 );

// Area - of a triangle in 3D
real Area( const vec3& v1, const vec3& v2, const vec3& v3 );

// Volume - of a tetrahedron in 3D
real Volume( const vec3& v1, const vec3& v2, const vec3& v3, const vec3& v4 );

// Cotangent - at vertex v1 of the triangle
real Cotangent( const vec2& v1, const vec2& v2, const vec2& v3 );

// PolygonArea - for arbitrary simple polygons
real PolygonArea( const std::vector<vec2>& verts );

// PolygonArea - for polygons in 3D
real PolygonArea( std::vector<vec3> verts );

// IsPointInTet
bool IsPointInTet( const vec3& pt, const vec3& v1, const vec3& v2, const vec3& v3, const vec3& v4 );

// BarycentricCoords - 2D
vec3 BarycentricCoords( const vec2& pt, const vec2& v1, const vec2& v2, const vec2& v3 );

// BarycentricCoords - 3D
vec4 BarycentricCoords( const vec3& pt, const vec3& v1, const vec3& v2, const vec3& v3, const vec3& v4 );

// Affine - affine combination of three points
vec3 Affine( const vec3& weights, const vec3& p1, const vec3& p2, const vec3& p3 );

// Affine - affine combination of four points
vec3 Affine( const vec4& weights, const vec3& p1, const vec3& p2, const vec3& p3, const vec3& p4 );


#endif  // _GEOMETRY_H_