#ifndef _LOG_FUNCTIONS_H_
#define _LOG_FUNCTIONS_H_

#include "util/log/log_impl.h"

namespace ns_log {

///////////////////////////////////////////////////////////////////////////////////////////////////////
// Log functions

#ifdef WIN32
void write_to_dbg_wnd(const std::string &, const std::string &msg);
#endif

void write_to_cout(const std::string &, const std::string &msg);

struct write_to_file
{
    write_to_file(const std::string & filename, bool overwrite);
    void operator()(const std::string&, const std::string & msg) const;
private:
    std::string filename;
};


/* 
Represents an array of appenders. It forwards each received message to all of its
underlying appenders
*/
struct appender_array
{
    appender_array& add( appender_func func )
    {
        m_funcs.push_back(func);
        return *this;
    }

    void operator()( const std::string & log_name, const std::string & msg );

private:
    std::vector<appender_func> m_funcs;
};


///////////////////////////////////////////////////////////////////////////////////////////////////////
// Modifier functions

// writes the prefix
void prepend_prefix(const std::string &prefix, std::string & msg);
// appends an enter if not already found
void append_enter(const std::string &, std::string & msg);
// prefixes the message with the time
void prepend_time(const std::string &, std::string & msg);



}  // namespace ns_log


#endif  // _LOG_FUNCTIONS_H_
