#include "util/log/log.h"
#include "util/log/log_impl.h"
#include <sstream>
#include <algorithm>

namespace ns_log {

logger_impl* find_log_by_name(const std::string & log_name);

logger::logger(const std::string & log_name)
{
    m_impl = find_log_by_name( log_name );
}

logger::~logger()
{
    // empty
}

std::string logger::name() const
{
    return m_impl->name;
}

bool logger::is_enabled() const
{
    return m_impl->is_enabled();
}

void logger::write_msg(const std::string & msg)
{
    if( !m_impl->is_enabled() )
        return;

    std::string copy = msg;

    // run modifiers
    for( std::vector<modifier_func>::const_iterator iter = m_impl->modifiers().begin(); iter != m_impl->modifiers().end(); ++iter )
        (*iter)( m_impl->name, copy );

    // run appenders
    for( std::vector<appender_func>::const_iterator iter = m_impl->appenders().begin(); iter != m_impl->appenders().end(); ++iter )
        (*iter)( m_impl->name, copy );
}

}  // namespace ns_log
